/**
 * @file
 * Defines behaviors for the Adyen Card Component payment method form.
 */

(function ($, Drupal, drupalSettings, AdyenCheckout) {
  /**
   * Attaches the commerceAdyenCC behavior.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches the commerceAdyenCC behavior.
   */
  Drupal.behaviors.commerceAdyenCC = {
    attach(context) {
      const $form = $(".adyen-cc-form", context)
        .closest("form")
        .once("adyen-cc-attach");
      if ($form.length === 0) {
        return;
      }
      const $submit = $form.find(":input.button--primary");
      const $verify = $form.find(":input[name=verify_payment]");

      $submit.attr("disabled", true);
      $verify.attr("disabled", true);

      // Form submit.
      $form.on("submit.adyen_cc", (event, options) => {
        // Disable the submit button to prevent repeated clicks.
        $form.find(".button--primary").prop("disabled", true);
        // Disable other form buttons, so that they don't cause an inadvertent
        // form submission or ajax request.
        Drupal.behaviors.commerceAdyenCC.formButtons = $form.find(":submit:not([disabled]):not(.button--primary):not([name=verify_payment])");
        Drupal.behaviors.commerceAdyenCC.formButtons.prop("disabled", true);
        options = options || {};
        if (options.populated) {
          return;
        }
        event.preventDefault();

        $verify.trigger("mousedown");

        // Prevent the form from submitting with the default action.
        return false;
      });

      const waitForSdk = setInterval(async () => {
        if ((typeof AdyenCheckout !== "undefined") && (typeof Drupal.commerceAdyenWebComponent !== 'undefined')) {
          let commerceAdyenCheckout = {};
          commerceAdyenCheckout = await Drupal.commerceAdyenWebComponent(
            $form,
            drupalSettings.commerceAdyenCC
          );
          $form.data("adyen-cc", commerceAdyenCheckout);
          clearInterval(waitForSdk);
        }
      }, 100);

    },
    detach(context, settings, trigger) {
      // Detaching on the wrong trigger will clear the Adyen form
      // on #ajax.
      if (trigger !== "unload") {
        return;
      }
      const $form = $(".adyen-cc-form", context).closest("form");
      if ($form.length === 0) {
        return;
      }
      const $submit = $form.find(":input.button--primary");

      $form.removeOnce("adyen-cc-attach");
      $submit.prop("disabled", false);
    }
  };

  $.extend(
    Drupal.theme,
    /** @lends Drupal.theme */ {
      commerceAdyenCCError(message) {
        return $(
          `${'<div role="alert">' +
            '<div class="messages messages--error">'}${message}</div>` +
            `</div>`
        );
      }
    }
  );

})(jQuery, Drupal, drupalSettings, window.AdyenCheckout);
